<?php

/**
 * Webkul Software.
 *
 * @category  Webkul
 * @package   Webkul_Eshopsync
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Controller\Adminhtml\AccountMap;

use Magento\Backend\App\Action\Context;
use Magento\Framework\View\Result\PageFactory;

class Index extends \Magento\Backend\App\Action
{
    /** @var \Magento\Framework\View\Result\PageFactory */
    protected $_resultPageFactory;

    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory
    ) {

        parent::__construct($context);
        $this->_resultPageFactory = $resultPageFactory;
    }

    public function execute()
    {   
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $magentoMessenger = $objectManager->create('\Magento\Framework\Message\ManagerInterface');
        $magentoMetaData = $objectManager->create('\Magento\Framework\App\ProductMetadataInterface');
        $data = $objectManager->create('\Webkul\SalesforceConnector\Helper\Data');
        $appEdition = $data->getConfigValue('general', 'mgedition');
        $fieldmap = $data->getConfigValue('general', 'fieldmap');
        if(!$fieldmap){
            return '';
        }
        if (($magentoMetaData->getEdition() == 'Community' && $appEdition == 1) || ($magentoMetaData->getEdition() == 'Enterprise' && $appEdition == 0)) {
            $magentoMessenger->addError(($appEdition == 0) ? "you seem to be using Community edition of this extension but have Enterprise edition of Magento. Kindly install the correct version of SalesforceConnector" : "you seem to be using Enterprise edition of this extension but have community edition of Magento. Kindly install the correct version of SalesforceConnector");
            $resultRedirect = $this->resultRedirectFactory->create();
            $resultRedirect->setPath('adminhtml/system_config/edit/section/sfConnect/');
            return $resultRedirect;
        }
        $resultPage = $this->_resultPageFactory->create();
        $resultPage->getConfig()->getTitle()->prepend(__('Salesforce Account Field Mapping'));
        return $resultPage;
    }

    /*
     * Check permission via ACL resource
     */
    // protected function _isAllowed()
    // {
    //     return $this->_authorization->isAllowed('Webkul_Eshopsync::customers_index');
    // }
}
