<?php

/**
 * Webkul Software.
 *
 * @category  Webkul
 *
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Helper;

// use Exception;
use Magento\Catalog\Model\Product as ProductModel;
use Magento\CatalogInventory\Api\Data\StockItemInterface;
use Magento\CatalogInventory\Model\Stock\Item;
use Magento\CatalogInventory\Model\StockRegistry;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Store\Model\Store;
use stdClass;

/**
 * Eshopsync category helper.
 */
class Product extends AbstractHelper
{
    protected $_storeManager;
    protected $_directory;
    protected $_eventManager;
    protected $_json;
    protected $_resourceConfig;
    protected $_image;
    protected $_eshopsyncCategory;
    protected $_eshopsyncProduct;
    protected $_helperData;
    protected $_helperConnection;
    protected $_helperCategory;

    /**
     * @var StockRegistry
     */
    protected $stockRegistry;

    /**
     * Product constructor.
     *
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\App\Filesystem\DirectoryList $directory
     * @param \Magento\Framework\Event\Manager $eventManager
     * @param \Magento\Framework\Json\Helper\Data $jsonHelper
     * @param \Magento\Config\Model\ResourceModel\Config $resourceConfig
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Catalog\Model\ProductFactory $product
     * @param \Magento\Catalog\Helper\Image $image
     * @param \Webkul\SalesforceConnector\Model\CategoryFactory $eshopsyncCategory
     * @param \Webkul\SalesforceConnector\Model\ProductFactory $eshopsyncProduct
     * @param Data $helperData
     * @param Connection $helperConnection
     * @param Category $helperCategory
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\App\Filesystem\DirectoryList $directory,
        \Magento\Framework\Event\Manager $eventManager,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Magento\Config\Model\ResourceModel\Config $resourceConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ProductFactory $product,
        \Magento\Catalog\Helper\Image $image,
        \Webkul\SalesforceConnector\Model\CategoryFactory $eshopsyncCategory,
        \Webkul\SalesforceConnector\Model\ProductFactory $eshopsyncProduct,
        \Webkul\SalesforceConnector\Helper\Data $helperData,
        \Webkul\SalesforceConnector\Helper\Connection $helperConnection,
        \Webkul\SalesforceConnector\Helper\Category $helperCategory,
        StockRegistry $stockRegistry
    ) {
        $this->_storeManager = $storeManager;
        $this->_directory = $directory;
        $this->_eventManager = $eventManager;
        $this->_json = $jsonHelper;
        $this->_resourceConfig = $resourceConfig;
        $this->_catalogProduct = $product;
        $this->_image = $image;
        $this->_eshopsyncCategory = $eshopsyncCategory;
        $this->_eshopsyncProduct = $eshopsyncProduct;
        $this->_helperData = $helperData;
        $this->_helperConnection = $helperConnection;
        $this->_helperCategory = $helperCategory;
        $this->stockRegistry = $stockRegistry;

        parent::__construct($context);
    }

    public static $product_type = [
        'simple' => 'Simple Product',
        'grouped' => 'Grouped Product',
        'configurable' => 'Configurable Product',
        'virtual' => 'Virtual Product',
        'bundle' => 'Bundle Product',
        'downloadable' => 'Downloadable Product',
    ];

    public function productMapping($data)
    {
        $model = $this->_eshopsyncProduct->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $data['magento_id'])->getData();
        if (count($collection) == 1) {
            $data['entity_id'] = $collection[0]['entity_id'];
        } elseif (count($collection) > 1) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
        $model->setData($data);
        $model->save();
    }

    public function deleteMapping($id)
    {
        $model = $this->_eshopsyncProduct->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $id)->getData();
        if (count($collection) > 0) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
    }

    /**
     * @param stdClass $sObject
     * @param ProductModel $product
     */
    public function addProductFieldMap(
        stdClass $sObject,
        ProductModel $product
    ) {
        $productFieldMap = $this->_helperData->getConfigValue('acc', 'productFieldMap');
        if (is_array($productFieldMap) && count($productFieldMap) > 0) {
            /** @var StockItemInterface|Item $stockItem */
            $stockItem = $this->stockRegistry->getStockItem($product->getId());
            $stockItemData = $stockItem->getData();
            $stockFields = array_keys($stockItemData);

            foreach ($productFieldMap as $field) {
                $magentoField = $field['magentoField'];
                $fieldCall = 'get' . $magentoField;
                $sfField = $field['saleForceField'];

                if (in_array($field['magentoField'], $stockFields)) {
                    $sObject->{$sfField} = $stockItem->{$fieldCall}();

                    continue;
                }

                $sObject->$sfField = $product->{$fieldCall}();
            }
        }
    }

    public function syncSpecificProduct($productId)
    {
        $sfPdtId = $imageExist = false;
        $reqArr = $mediaLink = [];
        $product = $this->_catalogProduct->create()->load($productId);
        $status = $product->getStatus();
        if ($status == '2') {
            $status = false;
        } else {
            $status = true;
        }
        $sObject = new stdClass();
        $type = $product->getTypeID();
        if (isset(self::$product_type[$type])) {
            $sObject->webkul_es_mage__Magento_Product_Type__c = self::$product_type[$type];
        }
        $name = $product->getName();
        /*salesforce product object creation*/
        $sObject->IsActive = $status;
        $sObject->Name = $name;
        $productDescriptionWithoutHTMLTags = strip_tags($product->getDescription());
        $sObject->Product_Description_Long__c = $productDescriptionWithoutHTMLTags;
        // $sObject->webkul_es_mage__Magento_Product_ID__c = $productId;
        $sObject->webkul_es_mage__Magento_Product_Sku__c = $product->getSku();
        if ($product->getWeight()) {
            $sObject->webkul_es_mage__Magento_Product_Weight__c = $product->getWeight();
        }
        $sObject->webkul_es_mage__Magento_Meta_Titile__c = $product->getMetaTitle();
        $sObject->webkul_es_mage__Magento_Meta_Keywords__c = $product->getMetaKeyword();
        $sObject->webkul_es_mage__Magento_Meta_Descritption__c = $product->getMetaDescription();
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $productStockObj = $objectManager
            ->get(\Magento\CatalogInventory\Api\StockRegistryInterface::class)
            ->getStockItem($productId);
        $sObject->webkul_es_mage__Magento_Stock_Availability__c = ($productStockObj->getQty()>0)?'In Stock':'Out of Stock';
        $sObject->webkul_es_mage__Magento_Short_Description__c = $product->getShortDescription();

        $this->addProductFieldMap($sObject, $product);

        try {

            if ($product->getImage()) {

                $baseUrl = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
                $image_path = $baseUrl . 'catalog/product' . $product->getImage();
                $pathDetail = pathinfo($image_path);
                if ($image_path) {

                    $imageId = $this
                        ->_helperData
                        ->uploadImageToSalesForceFile('product', $productId, $image_path, $reqArr);

                    // if ($productId == 263) {
                    //     echo"<pre>";var_dump($imageId);die;
                    // }
                    if ($imageId === true) {
                        $sObject->webkul_es_mage__Magento_Image_ID__c = '@{image_' . $productId . '.id}';
                    } elseif (!empty($imageId)) {
                        $imageExist = true;
                        $sObject->webkul_es_mage__Magento_Image_ID__c = $imageId;
                    }
                }
            }
        } catch (\Exception $e) {
            $this->_helperData->eshopsyncLog($e->getMessage());
        }
        try {
            $reqArr[] = [
                'method' => 'PATCH',
                'referenceId' => $productId,
                'url' => '/services/data/v46.0/sobjects/Product2/webkul_es_mage__Magento_Product_ID__c/' . $productId,
                'body' => $sObject,
            ];
            if ($this->_helperData->getConfigValue('catPdt', 'imgToFile')) {
                $reqArr[] = [
                    'method' => 'GET',
                    'referenceId' => 'contDoc',
                    'url' => $imageExist ?
                        "/services/data/v46.0/query/?q=SELECT+ContentDocumentId+FROM+ContentVersion+WHERE+Id+=+'" .
                        $imageId .
                        "'" :
                        "/services/data/v46.0/query/?q=SELECT+ContentDocumentId+FROM+ContentVersion+WHERE+Id+=+'@{image_" .
                        $productId .
                        ".id}'",
                ];
                $reqArr[] = [
                    'method' => 'POST',
                    'referenceId' => 'imagelink_' . $productId,
                    'url' => '/services/data/v46.0/sobjects/ContentDocumentLink',
                    'body' => [
                        'LinkedEntityId' => '@{' . $productId . '.id}',
                        'ShareType' => 'I',
                        'Visibility' => 'AllUsers',
                        'ContentDocumentId' => '@{contDoc.records[0].ContentDocumentId}',
                    ],
                ];
            }
            $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $reqArr]);

            //echo"<pre>";var_dump($reqArr,$response);die;
            foreach ($response['compositeResponse'] as $result) {
                if ($result['referenceId'] == $productId) {
                    $sfCatId = 0;
                    if (isset($result['body']['success']) && $result['body']['success']) {
                        $sfPdtId = $result['body']['id'];
                        $mappingData = [
                            'magento_id' => $productId,
                            'sforce_id' => $sfPdtId,
                            'created_by' => $this->_helperData->getCurrentUser(),
                            'sync_status' => 1,
                            'error' => '',
                        ];
                        /* Call for category Mapping */
                        $existingMapping = $catMappinReqArr = $unsetKeys = [];
                        $syncCate = $this->_helperData->getConfigValue('catPdt', 'synCat');
                        if ($syncCate) {
                            $pdtCategories = $product->getCategoryIds();
                            $catMappingData = $this
                                ->_helperData
                                ->fetchMappingDetails($this->_eshopsyncCategory, $pdtCategories, true);
                            if (!$result['body']['created']) {
                                $existingMapping = $this->_helperConnection->getSObject(
                                    "SELECT Id,webkul_es_mage__categories__c 
                                    FROM webkul_es_mage__product_categories__c 
                                    WHERE webkul_es_mage__Product__c = '" . $sfPdtId . "'",
                                    0
                                );
                            }
                            try {
                                foreach ($pdtCategories as $category) {
                                    if (($key = array_search($category, array_column($catMappingData, 'magento_id'))) !== false) {
                                        $sfCatId = $catMappingData[$key]['sforce_id'];
                                        /* create mapping entry */

                                        if ($result['body']['created'] || (count($existingMapping) == 0) || (is_array($existingMapping) && count($existingMapping) > 0 && (($mapExist = array_search($sfCatId, array_column($existingMapping, 'webkul_es_mage__categories__c'))) === false))) {
                                            $catMappinReqArr[] = [
                                                'method' => 'POST',
                                                'referenceId' => 'catMap_' . $sfCatId,
                                                'url' => '/services/data/v46.0/sobjects/webkul_es_mage__product_categories__c/',
                                                'body' => [
                                                    'webkul_es_mage__categories__c' => $sfCatId,
                                                    'webkul_es_mage__Product__c' => $sfPdtId,
                                                ],
                                            ];
                                        } else {
                                            $mapExist = array_search($sfCatId, array_column($existingMapping, 'webkul_es_mage__categories__c'));

                                            if ($mapExist !== false) {
                                                $unsetKeys[] = $mapExist;
                                            }
                                            /* unset key which is  related to existing category and delete rest of mapping entries*/
                                        }
                                    } else {
                                        /* sync cate thn make entry */
                                        $catArr = $this->_helperCategory->getCategoryArray($category);
                                        foreach ($catArr as $cat) {
                                            $catMappinReqArr[] = $cat;
                                        }
                                        // why check for sfcat id  when we need to create category?
                                        // if ($sfCatId) {
                                        $catMappinReqArr[] = [
                                            'method' => 'POST',
                                            'referenceId' => 'catMap_' . $sfCatId,
                                            'url' => '/services/data/v46.0/sobjects/webkul_es_mage__product_categories__c/',
                                            'body' => [
                                                'webkul_es_mage__categories__c' => '@{cat_' . $category . '.id}',
                                                'webkul_es_mage__Product__c' => $sfPdtId,
                                            ],
                                        ];
                                        // }
                                    }
                                }
                            } catch (\Exception $e) {
                                $this->_helperData->eshopsyncLog($e->getMessage());
                            }
                            if (count($catMappinReqArr) > 0) {
                                $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $catMappinReqArr]);
                                if ($response && count($response) > 0) {
                                    foreach ($response['compositeResponse'] as $resData) {
                                        if (strpos($resData['referenceId'], 'cat_') !== false && isset($resData['body']['success']) && $resData['body']['success']) {
                                            $catId = ltrim($resData['referenceId'], 'cat_');
                                            $catMappingData = [
                                                'magento_id' => $catId,
                                                'sforce_id' => $resData['body']['id'],
                                                'created_by' => $this->_helperData->getCurrentUser(),
                                                'sync_status' => 1,
                                                'error' => '',
                                            ];
                                            $this->_helperCategory->categoryMapping($catMappingData);
                                        }
                                    }
                                }
                            }
                            if (count($unsetKeys)) {
                                foreach ($unsetKeys as $K) {
                                    unset($existingMapping[$K]);
                                }
                            }
                            if (isset($existingMapping) && count($existingMapping) > 0) {
                                $ids = implode(',', array_column($existingMapping, 'Id'));
                                $url = '/services/data/v46.0/composite/sobjects?ids=' . $ids;
                                $response = $this->_helperConnection->deleteSobject($url);
                            }
                        }
                        /* Call for Procebook Entry */
                        try {
                            $priceBookEntryArr = [];
                            $priceBookSame = false;
                            $search = array_search(1, array_column($this->_helperConnection->_sfconfigData['pricebook'], 'standard'));
                            $standardPricebook = $this->_helperConnection->_sfconfigData['pricebook'][$search]['value'];

                            $priceBook = $this->_helperData->getConfigValue('catPdt', 'pricebook');
                            if(isset($priceBook) && $priceBook){
                                $selected_pricebook_id = $this->_json->jsonDecode($priceBook)['id'];
                            }
                            else{
                                $selected_pricebook_id = '';
                            }


                            if ($selected_pricebook_id == $standardPricebook) {
                                $priceBookSame = true;
                            }
                            if (!$result['body']['created']) {
                                $existingMapping = $this->_helperConnection->getSObject("SELECT Id,Pricebook2Id FROM PricebookEntry WHERE Product2Id = '" . $sfPdtId . "'", 0);
                            }
                            $price = $product->getPrice();
                            if ($type == 'configurable') {
                                $price = $product->getFinalPrice();
                            }
                            if ($type == 'bundle') {
                                $price = 0;
                            }
                            if (isset($standardPricebook) && !empty($standardPricebook) && $standardPricebook) {
                                if ($result['body']['created'] || (count($existingMapping) == 0) || (count($existingMapping) > 0 && ($search = array_search($standardPricebook, array_column($existingMapping, 'Pricebook2Id'))) === false)) {
                                    /* make entry */
                                    $priceBookEntryArr[] = [
                                        'method' => 'POST',
                                        'referenceId' => 'priceMap_' . $standardPricebook,
                                        'url' => '/services/data/v46.0/sobjects/PricebookEntry/',
                                        'body' => [
                                            'Product2Id' => $sfPdtId,
                                            'Pricebook2Id' => $standardPricebook,
                                            'UnitPrice' => $price,
                                            'IsActive' => 1,
                                        ],
                                    ];
                                } else {
                                    $search = array_search($standardPricebook, array_column($existingMapping, 'Pricebook2Id'));
                                    $entryId = $existingMapping[$search]['Id'];
                                    unset($existingMapping[$search]);
                                    $existingMapping = array_values($existingMapping);
                                    $priceBookEntryArr[] = [
                                        'method' => 'PATCH',
                                        'referenceId' => 'priceMap_' . $standardPricebook,
                                        'url' => '/services/data/v46.0/sobjects/PricebookEntry/' . $entryId,
                                        'body' => [
                                            // 'Product2Id' => $sfPdtId,
                                            // 'Pricebook2Id' => $standardPricebook,
                                            'IsActive' => 1,
                                            'UnitPrice' => $price,
                                        ],
                                    ];
                                }
                            }
                            if (!$priceBookSame) {
                                if ($result['body']['created'] || (isset($existingMapping) && count($existingMapping) == 0) || (isset($existingMapping) && count($existingMapping) > 0 && ($search = array_search($selected_pricebook_id, array_column($existingMapping, 'Pricebook2Id'))) === false)) {
                                    /* make entry for both pricebook id */
                                    $priceBookEntryArr[] = [
                                        'method' => 'POST',
                                        'referenceId' => 'priceMap_' . $selected_pricebook_id,
                                        'url' => '/services/data/v46.0/sobjects/PricebookEntry/',
                                        'body' => [
                                            'Product2Id' => $sfPdtId,
                                            'Pricebook2Id' => $selected_pricebook_id,
                                            'UnitPrice' => $price,
                                            'IsActive' => 1,
                                        ],
                                    ];
                                } else {
                                    $entryId = $existingMapping[$search]['Id'];
                                    unset($existingMapping[$search]);
                                    $existingMapping = array_values($existingMapping);

                                    $priceBookEntryArr[] = [
                                        'method' => 'PATCH',
                                        'referenceId' => 'priceMap_' . $selected_pricebook_id,
                                        'url' => '/services/data/v46.0/sobjects/PricebookEntry/' . $entryId,
                                        'body' => [
                                            // 'Product2Id' => $sfPdtId,
                                            // 'Pricebook2Id' => $selected_pricebook_id,
                                            'IsActive' => 1,
                                            'UnitPrice' => $price,
                                        ],
                                    ];
                                }
                            }

                            /* make request for entry */
                            if (count($priceBookEntryArr) > 0) {
                                $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $priceBookEntryArr]);
                            }
                            /* delete extra price mapping */
                            if (count($existingMapping) > 0) {
                                $ids = implode(',', array_column($existingMapping, 'Id'));
                                $response = $this->_helperConnection->compositeUpsertSObject([
                                    'compositeRequest' => [
                                        'method' => 'DELETE',
                                        'referenceId' => 'delMapping_' . $sfCatId,
                                        'url' => '/services/data/v46.0/composite/sobjects?ids=' . $ids,
                                    ],
                                ]);
                            }
                        } catch (\Exception $e) {
                            $this->_helperData->eshopsyncLog($e->getMessage());
                        }
                    } elseif (isset($result['body'][0]['errorCode'])) {
                        $mappingData = [
                            'magento_id' => $productId,
                            'sforce_id' => '',
                            'created_by' => $this->_helperData->getCurrentUser(),
                            'sync_status' => 0,
                            'error' => $result['body'][0]['message'],
                        ];
                    }
                    $this->productMapping($mappingData);
                }
            }
        } catch (\Exception $e) {
            $error = ' Error, Product Id ' . $productId . ' >>' . $e->getMessage();
            // Mage::log($error, null, 'eshopsync_connector.log');
            // $this->_logger->critical($error);
            $this->_helperData->eshopsyncLog($error);
        }

        return $sfPdtId;
    }

    public function exportProducts($pdtIds)
    {
        $success = $error = 0;
        $reqArr = $productArr = $imageArr = $insert_data = $catArr = $priceArr = [];
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $productCollection = $objectManager->create(
            \Magento\Catalog\Model\ResourceModel\Product\Collection::class
        );
        $productCollection->addAttributeToSelect('*')
            ->addFieldToFilter('entity_id', ['in' => $pdtIds])
            ->addStoreFilter(Store::DEFAULT_STORE_ID);
        foreach ($productCollection->getItems() as $product) {
            $status = $product->getStatus();
            if ($status == '2') {
                $status = false;
            } else {
                $status = true;
            }
            $sObject = new stdClass();
            $type = $product->getTypeID();
            if (isset(self::$product_type[$type])) {
                $sObject->webkul_es_mage__Magento_Product_Type__c = self::$product_type[$type];
            }
            $name = $product->getName();
            /*salesforce product object creation*/
            $sObject->IsActive = $status;
            $sObject->Name = $name;
            $productDescriptionWithoutHTMLTags = strip_tags($product->getDescription());
            $sObject->Product_Description_Long__c = $productDescriptionWithoutHTMLTags;
            // $sObject->webkul_es_mage__Magento_Product_ID__c = $productId;
            $sObject->webkul_es_mage__Magento_Product_Sku__c = $product->getSku();
            if ($product->getWeight()) {
                $sObject->webkul_es_mage__Magento_Product_Weight__c = $product->getWeight();
            }
            $sObject->webkul_es_mage__Magento_Meta_Titile__c = $product->getMetaTitle();
            $sObject->webkul_es_mage__Magento_Meta_Keywords__c = $product->getMetaKeyword();
            $sObject->webkul_es_mage__Magento_Meta_Descritption__c = $product->getMetaDescription();

            $productStockObj = $objectManager->get(\Magento\CatalogInventory\Api\StockRegistryInterface::class)->getStockItem($product['entity_id']);

            $sObject->webkul_es_mage__Magento_Stock_Availability__c = ($productStockObj->getQty()>0)?'In Stock':'Out of Stock';
            $sObject->webkul_es_mage__Magento_Short_Description__c = $product->getShortDescription();

            $this->addProductFieldMap($sObject, $product);

            if ($product->getImage()) {
                $baseUrl = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
                $image_path = $baseUrl . 'catalog/product' . $product->getImage();
                $imageArr[$product['entity_id']] = pathinfo($image_path);
                $productArr[$product['entity_id']] = $sObject;
            } else {
                $reqArr[] = [
                    'method' => 'PATCH',
                    'referenceId' => 'pdt_' . $product['entity_id'],
                    'url' => '/services/data/v46.0/sobjects/Product2/webkul_es_mage__Magento_Product_ID__c/' . $product['entity_id'],
                    'body' => $sObject,
                ];
            }
            $catArr[$product['entity_id']] = $product->getCategoryIds();
            $price = $product->getPrice();
            if ($type == 'configurable') {
                $price = $product->getFinalPrice();
            }
            if ($type == 'bundle') {
                $price = 0;
            }
            $priceArr[$product['entity_id']] = $price;
        }
        $syncToFile = $this->_helperData->getConfigValue('catPdt', 'imgToFile');

        // context for fecth file
        $fileContext = stream_context_create([
            'http' => [
                'ignore_errors' => true,
            ],
        ]);

        try {
            if (count($imageArr) > 0) {
                if($syncToFile){
                    $existingFiles = $this->_helperData->checkMultiFileExist(array_column($imageArr, 'basename'));
                    foreach ($imageArr as $pId => $image) {

                        $sfFileId = array_search($image['basename'], $existingFiles);
                        if ($sfFileId) {
                            $imageExist = $sfFileId;
                            $productArr[$pId]->webkul_es_mage__Magento_Image_ID__c = $sfFileId;
                        } else {
                            $imageExist = false;
                            $image_path = $image['dirname'] . '/' . $image['basename'];
                            $mediaImage = [];
                            $mediaImage['Title'] = $image['basename'];
                            $mediaImage['PathOnClient'] = $image_path;
                            $mediaImage['IsMajorVersion'] = false;
                            $mediaImage['VersionData'] = base64_encode(file_get_contents($image_path, false, $fileContext));
                            $reqArr[] = [
                                'method' => 'POST',
                                'referenceId' => 'image_' . $pId,
                                'url' => '/services/data/v46.0/sobjects/ContentVersion',
                                'body' => $mediaImage,
                            ];
                            $sObject = $productArr[$pId];
                            $sObject->webkul_es_mage__Magento_Image_ID__c = '@{image_' . $pId . '.id}';
                        }
                        $reqArr[] = [
                            'method' => 'PATCH',
                            'referenceId' => 'pdt_' . $pId,
                            'url' => '/services/data/v46.0/sobjects/Product2/webkul_es_mage__Magento_Product_ID__c/' . $pId,
                            'body' => $sObject,
                        ];
                        $reqArr[] = [
                            'method' => 'GET',
                            'referenceId' => 'contDoc',
                            'url' => $imageExist ? "/services/data/v46.0/query/?q=SELECT+ContentDocumentId+FROM+ContentVersion+WHERE+Id+=+'" . $imageExist . "'" : "/services/data/v46.0/query/?q=SELECT+ContentDocumentId+FROM+ContentVersion+WHERE+Id+=+'@{image_" . $pId . ".id}'",
                        ];
                        $reqArr[] = [
                            'method' => 'POST',
                            'referenceId' => 'imagelink_' . $pId,
                            'url' => '/services/data/v46.0/sobjects/ContentDocumentLink',
                            'body' => [
                                'LinkedEntityId' => '@{pdt_' . $pId . '.id}',
                                'ShareType' => 'I',
                                'Visibility' => 'AllUsers',
                                'ContentDocumentId' => '@{contDoc.records[0].ContentDocumentId}',
                            ],
                        ];
                    }
                } else {

                    $existingDocs = $this->_helperData->checkMultiDocExist(array_column(
                        $imageArr,
                        'basename'
                    ));
                    $existingFiles = array_column($existingDocs, 'Name', 'Id');
                    foreach ($imageArr as $pId => $image) {
                        $image_path = $image['dirname'] . '/' . $image['basename'];
                        $documentMedia = new stdClass();
                        $name = $image['basename'];
                        $documentMedia->Name = $name;
                        $documentMedia->IsPublic = true;

                        $docFolder = $this->_helperData->getConfigValue('catPdt', 'docFolder');
                        if(!(isset($docFolder) && $docFolder!=''))
                            return [$success, $error, 'error'=>"No Doc Folder Set Please check Configuration."];
                        $documentMedia->FolderId = $this->_json->jsonDecode($docFolder)['id'];
                        $documentMedia->Body = base64_encode(file_get_contents($image_path, false, $fileContext));
                        $sfFileId = array_search($image['basename'], $existingFiles);
                        if ($sfFileId) {
                            $reqArr[] = [
                                'method' => 'PATCH',
                                'referenceId' => 'image_' . $pId,
                                'url' => '/services/data/v46.0/sobjects/Document/Id/' . $sfFileId,
                                'body' => $documentMedia,
                            ];
                        } else {
                            $reqArr[] = [
                                'method' => 'POST',
                                'referenceId' => 'image_' . $pId,
                                'url' => '/services/data/v46.0/sobjects/Document/',
                                'body' => $documentMedia,
                            ];
                        }
                        $sObject = $productArr[$pId];
                        $sObject->webkul_es_mage__Magento_Image_ID__c = '@{image_' . $pId . '.id}';
                        $reqArr[] = [
                            'method' => 'PATCH',
                            'referenceId' => 'pdt_' . $pId,
                            'url' => '/services/data/v46.0/sobjects/Product2/webkul_es_mage__Magento_Product_ID__c/' . $pId,
                            'body' => $sObject,
                        ];
                    }
                }
            }
        } catch (\Exception $e) {
            $this->_helperData->eshopsyncLog($e->getMessage());
        }

        if (count($reqArr) > 0) {
            $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $reqArr]);
            // echo "<pre>";
            // print_r($response);
            // die;
            // sync cat
            $syncCate = $this->_helperData->getConfigValue('catPdt', 'synCat');
            $reqArr = [];
            foreach ($response['compositeResponse'] as $responseData) {
                if (strpos($responseData['referenceId'], 'pdt_') !== false && isset($responseData['body']['success']) && $responseData['body']['success']) {
                    $pid = ltrim($responseData['referenceId'], 'pdt_');
                    ++$success;
                    $sfPdtId = $responseData['body']['id'];
                    if ($responseData['body']['created']) {
                        $insert_data[] = [
                            'magento_id' => $pid,
                            'sforce_id' => $sfPdtId,
                            'created_by' => $this->_helperData->getCurrentUser(),
                            'sync_status' => 1,
                            'error' => '',
                        ];
                        if ($syncCate) {
                            $this->productCatSync($pid, $catArr[$pid], $sfPdtId, $reqArr, true);
                        }
                        $this->productpriceBookEntry($pid, $priceArr[$pid], $sfPdtId, $reqArr, true);
                    } else {
                        $mappingData = [
                            'magento_id' => $pid,
                            'sforce_id' => $sfPdtId,
                            'created_by' => $this->_helperData->getCurrentUser(),
                            'sync_status' => 1,
                            'error' => '',
                        ];
                        $this->productMapping($mappingData);
                        if ($syncCate) {
                            $this->productCatSync($pid, $catArr[$pid], $sfPdtId, $reqArr);
                        }

                        $this->productpriceBookEntry($pid, $priceArr[$pid], $sfPdtId, $reqArr);
                    }
                } elseif (strpos($responseData['referenceId'], 'pdt_') !== false && isset($responseData['body'][0]['errorCode'])) {
                    $pid = ltrim($responseData['referenceId'], 'pdt_');
                    ++$error;
                    $mappingData = [
                        'magento_id' => $pid,
                        'sforce_id' => '',
                        'created_by' => $this->_helperData->getCurrentUser(),
                        'sync_status' => 0,
                        'error' => $responseData['body'][0]['message'],
                    ];
                    $this->productMapping($mappingData);
                }
            }
            if (count($reqArr) > 0) {
                $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $reqArr]);
                if ($response && count($response) > 0) {
                    foreach ($response['compositeResponse'] as $resData) {
                        if (strpos($resData['referenceId'], 'cat_') !== false && isset($resData['body']['success']) && $resData['body']['success']) {
                            $catId = ltrim($resData['referenceId'], 'cat_');
                            $mappingData = [
                                'magento_id' => $catId,
                                'sforce_id' => $resData['body']['id'],
                                'created_by' => $this->_helperData->getCurrentUser(),
                                'sync_status' => 1,
                                'error' => '',
                            ];
                            $this->_helperCategory->categoryMapping($mappingData);
                        }
                    }
                }
            }
            if (count($insert_data) > 0) {
                //bulk insert code
                $this->_helperData->bulkInsert('wk_salesforce_eshopsync_product_mapping', $insert_data);
            }
        }
        //return $success;
        return [$success, $error];
    }

    public function productCatSync($productId, $pdtCategories, $sfId, &$reqArr, $created = false)
    {
        $catMappingData = $this->_helperData->fetchMappingDetails($this->_eshopsyncCategory, $pdtCategories, true);
        if ($created) {
            foreach ($pdtCategories as $category) {
                if (($key = array_search($category, array_column($catMappingData, 'magento_id'))) !== false) {
                    //only create mapping entry
                    $sfCatId = $catMappingData[$key]['sforce_id'];
                    $reqArr[] = [
                        'method' => 'POST',
                        'referenceId' => 'catMap_' . $sfCatId,
                        'url' => '/services/data/v46.0/sobjects/webkul_es_mage__product_categories__c/',
                        'body' => [
                            'webkul_es_mage__categories__c' => $sfCatId,
                            'webkul_es_mage__Product__c' => $sfId,
                        ],
                    ];
                } else {
                    // sync cat first then create map
                    $catArr = $this->_helperCategory->getCategoryArray($category);
                    $reqArr = array_values($catArr);
                    // foreach($catArr as $cat)
                    //     $reqArr[] = $cat;
                    $reqArr[] = [
                        'method' => 'POST',
                        'referenceId' => 'catMap_' . $category,
                        'url' => '/services/data/v46.0/sobjects/webkul_es_mage__product_categories__c/',
                        'body' => [
                            'webkul_es_mage__categories__c' => '@{cat_' . $category . '.id}',
                            'webkul_es_mage__Product__c' => $sfId,
                        ],
                    ];
                }
            }
        } else {
            $existingMapping = $this->_helperConnection->getSObject("SELECT Id,webkul_es_mage__categories__c FROM webkul_es_mage__product_categories__c WHERE webkul_es_mage__Product__c = '" . $sfId . "'", 0);
            foreach ($pdtCategories as $category) {
                if (($key = array_search($category, array_column($catMappingData, 'magento_id'))) !== false) {
                    //only create mapping entry
                    $sfCatId = $catMappingData[$key]['sforce_id'];
                    if (($mapExist = array_search($sfCatId, array_column($existingMapping, 'webkul_es_mage__categories__c'))) === false) {
                        $reqArr[] = [
                            'method' => 'POST',
                            'referenceId' => 'catMap_' . $sfCatId,
                            'url' => '/services/data/v46.0/sobjects/webkul_es_mage__product_categories__c/',
                            'body' => [
                                'webkul_es_mage__categories__c' => $sfCatId,
                                'webkul_es_mage__Product__c' => $sfId,
                            ],
                        ];
                    }
                } else {
                    //sync cat and then create map
                    $catArr = $this->_helperCategory->getCategoryArray($category);
                    foreach ($catArr as $cat) {
                        $reqArr[] = $cat;
                    }
                    $reqArr[] = [
                        'method' => 'POST',
                        'referenceId' => 'catMap_' . $category,
                        'url' => '/services/data/v46.0/sobjects/webkul_es_mage__product_categories__c/',
                        'body' => [
                            'webkul_es_mage__categories__c' => '@{cat_' . $category . '.id}',
                            'webkul_es_mage__Product__c' => $sfId,
                        ],
                    ];
                }
            }
        }
    }

    public function productpriceBookEntry($productId, $price, $sfPdtId, &$reqArr, $created = false)
    {
        $search = array_search(1, array_column($this->_helperConnection->_sfconfigData['pricebook'], 'standard'));
        $standardPricebook = $this->_helperConnection->_sfconfigData['pricebook'][$search]['value'];

        $priceBook = $this->_helperData->getConfigValue('catPdt', 'pricebook');
        if(isset($priceBook) && $priceBook){
            $selected_pricebook_id = $this->_json->jsonDecode($priceBook)['id'];
        }
        else{
            $selected_pricebook_id = '';
        }
        if ($created) {
            $reqArr[] = [
                'method' => 'POST',
                'referenceId' => 'priceMapStand_' . $productId . $standardPricebook,
                'url' => '/services/data/v46.0/sobjects/PricebookEntry/',
                'body' => [
                    'Product2Id' => $sfPdtId,
                    'Pricebook2Id' => $standardPricebook,
                    'UnitPrice' => $price,
                    'IsActive' => 1,
                ],
            ];
            if ($selected_pricebook_id != $standardPricebook) {
                $reqArr[] = [
                    'method' => 'POST',
                    'referenceId' => 'priceMapSelected_' . $productId . $selected_pricebook_id,
                    'url' => '/services/data/v46.0/sobjects/PricebookEntry/',
                    'body' => [
                        'Product2Id' => $sfPdtId,
                        'Pricebook2Id' => $selected_pricebook_id,
                        'UnitPrice' => $price,
                        'IsActive' => 1,
                    ],
                ];
            }
        } else {
            $existingMapping = $this->_helperConnection->getSObject("SELECT Id,Pricebook2Id FROM PricebookEntry WHERE Product2Id = '" . $sfPdtId . "'", 0);
            $existingMapping = array_column($existingMapping, 'Id', 'Pricebook2Id');
            if (!isset($existingMapping[$standardPricebook])) {
                $reqArr[] = [
                    'method' => 'POST',
                    'referenceId' => 'priceMapStandard_' . $productId . $standardPricebook,
                    'url' => '/services/data/v46.0/sobjects/PricebookEntry/',
                    'body' => [
                        'Product2Id' => $sfPdtId,
                        'Pricebook2Id' => $standardPricebook,
                        'UnitPrice' => $price,
                        'IsActive' => 1,
                    ],
                ];
            } else {
                $sfEntryID = $existingMapping[$standardPricebook];
                $reqArr[] = [
                    'method' => 'PATCH',
                    'referenceId' => 'priceMapStan_' . $productId . $standardPricebook,
                    'url' => '/services/data/v46.0/sobjects/PricebookEntry/' . $sfEntryID,
                    'body' => [
                        // 'Product2Id' => $sfPdtId,
                        // 'Pricebook2Id' => $standardPricebook,
                        'IsActive' => 1,
                        'UnitPrice' => $price,
                    ],
                ];
            }
            if ($selected_pricebook_id != $standardPricebook) {
                if (!isset($existingMapping[$selected_pricebook_id])) {
                    $reqArr[] = [
                        'method' => 'POST',
                        'referenceId' => 'priceMapSelected_' . $productId . $selected_pricebook_id,
                        'url' => '/services/data/v46.0/sobjects/PricebookEntry/',
                        'body' => [
                            'Product2Id' => $sfPdtId,
                            'Pricebook2Id' => $selected_pricebook_id,
                            'UnitPrice' => $price,
                            'IsActive' => 1,
                        ],
                    ];
                } else {
                    $sfEntryID = $existingMapping[$selected_pricebook_id];
                    $reqArr[] = [
                        'method' => 'PATCH',
                        'referenceId' => 'priceMapSelected_' . $productId . $selected_pricebook_id,
                        'url' => '/services/data/v46.0/sobjects/PricebookEntry/' . $sfEntryID,
                        'body' => [
                            // 'Product2Id' => $sfPdtId,
                            // 'Pricebook2Id' => $selected_pricebook_id,
                            'IsActive' => 1,
                            'UnitPrice' => $price,
                        ],
                    ];
                }
            }
        }
    }
}
